"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""

from django.test import TestCase
from pizza.models import *
from pizza.facets import pizza
from django.db.models import Q
from sesql.longquery import longquery
from faceted_sesql.search import FacetedSearch

class PizzaTestCore(TestCase):
    fixtures = [ 'test' ]

    def shop(self, name):
        return Shop.objects.get(name=name)

    def pizza(self, name):
        return Pizza.objects.get(name=name)

class Test0Pizzas(PizzaTestCore):
    def test_initial(self):
        """
        Test that data look good in db
        """
        self.assertEqual(len(Pizza.objects.all()), 15)
        self.assertEqual(self.shop('Giant pizza').computed_opendays(),
                         (1,2,3,5,6))
        self.assertEqual(self.shop('Weird pizza').computed_opendays(),
                         (1,2,3,4,5,6))
        self.assertEqual(self.shop('Week-end pizza').computed_opendays(),
                         (6,7))
        self.assertEqual(self.shop('All days pizza').computed_opendays(),
                         (1,2,3,4,5,6,7))


        self.assertEqual(self.pizza('All cheese !').qualities,
                         (1,2,3))

    def test_sesql(self):
        """
        Ensure data is in SeSQL
        """
        pizza = Q(classname = Pizza)
        self.assertEqual(len(longquery(pizza)), 15)
        self.assertEqual(len(longquery(pizza & Q(opendays = 1))), 10)
        
class Test1Faceted(PizzaTestCore):
    def test_simple(self):
        """
        Test simple filtering
        """
        query = FacetedSearch(pizza)
        self.assertEqual(len(query), 15)
        query.set_criteria('crust', (1,))
        self.assertEqual(len(query), 9)
        query.set_criteria('crust', (1,4))
        self.assertEqual(len(query), 11)

    def test_multiple(self):
        """
        Test filtering on multi-valued fields
        """
        query = FacetedSearch(pizza)
        self.assertEqual(len(query), 15)
        query.set_criteria('opendays', (7,))
        self.assertEqual(len(query), 8)
        query.set_criteria('opendays', (4,7))
        self.assertEqual(len(query), 12)
        
    def test_combinated(self):
        """
        Test filtering on combinated values
        """
        query = FacetedSearch(pizza)
        self.assertEqual(len(query), 15)
        query.set_criteria('crust', (1,))
        self.assertEqual(len(query), 9)
        query.set_criteria('opendays', (7,))
        self.assertEqual(len(query), 6)
        query.set_criteria('crust', None)
        self.assertEqual(len(query), 8)
        query = FacetedSearch(pizza)
        self.assertEqual(len(query), 15)
        query.set_criteria('opendays', (7,))
        self.assertEqual(len(query), 8)
        query.set_criteria('crust', (1,))
        self.assertEqual(len(query), 6)
        query.set_criteria('crust', None)
        self.assertEqual(len(query), 8)
        query.set_criteria('toppings', (1,2,))
        self.assertEqual(len(query), 4)
        
    def test_precount(self):
        """
        Check the count of the number of options per value
        """
        query = FacetedSearch(pizza)
        self.assertEqual(len(query), 15)
        query.set_criteria('crust', (1,4))
        self.assertEqual(len(query), 11)
        self.assertEqual(query.data.facets['crust'].values[1], 9)
        self.assertEqual(query.data.facets['crust'].values[4], 2)
        query.set_criteria('crust', (1,))
        self.assertEqual(query.data.facets['opendays'].values[7], 6)
        query.set_criteria('opendays', (7,))
        self.assertEqual(query.data.facets['crust'].values[1], 6)
        self.assertEqual(query.data.facets['crust'].values[4], 2)

    def test_sort(self):
        """
        Check that sorting works well
        """
        query = FacetedSearch(pizza)
        query.set_criteria('toppings', (2,))
        query.set_order('asc_price')
        last = None
        for item in query:
            self.assertTrue(item.price >= last)
            last = item.price
        query.set_order('desc_weight')
        last = None
        for item in query:
            self.assertTrue(last is None or item.weight <= last)
            last = item.weight

    def test_external_filter(self):
        """
        Check that sorting works well
        """
        query = FacetedSearch(pizza)
        query.set_criteria('crust', (1,4))
        query.set_criteria('opendays', (7,))
        self.assertEqual(query.data.facets['crust'].values[1], 6)
        self.assertEqual(len(query), 8)
        query.apply_external_filter(lambda obj: len(obj.name) % 2 == 0)
        for item in query:
            self.assertEqual(len(item.name) % 2, 0)
        self.assertEqual(query.data.facets['crust'].values[1], 4)
        self.assertEqual(len(query), 5)

       
class Test2Edit(PizzaTestCore):
    def test_simple(self):
        """
        Test simple editing
        """
        query = FacetedSearch(pizza)
        self.assertEqual(len(query), 15)
        query.set_criteria('crust', (1,))
        self.assertEqual(len(query), 9)
        pizzas = list(query)
        pizzas[0].crust = 2
        pizzas[0].save()
        query = FacetedSearch(pizza)
        self.assertEqual(len(query), 15)
        query.set_criteria('crust', (1,))
        self.assertEqual(len(query), 8)

    def test_relations(self):
        """
        Test simple relation edition
        """
        query = FacetedSearch(pizza)
        self.assertEqual(len(query), 15)
        query.set_criteria('toppings', (2,))
        self.assertEqual(len(query), 7)
        pizzas = Pizza.objects.exclude(toppings__id = 2)
        pizzas[0].toppings.add(Topping.objects.get(pk = 2))
        pizzas[0].save()
        query = FacetedSearch(pizza)
        self.assertEqual(len(query), 15)
        query.set_criteria('toppings', (2,))
        self.assertEqual(len(query), 8)
